"use strict";

exports.__esModule = true;
exports.useRef = useRef;
exports.unrefElement = unrefElement;
exports.useDebouncedRef = useDebouncedRef;

var _vue = require("vue");

var _timers = require("./timers");

/**
 * For internal use
 *
 * Creates refs that will be bound to the template/render function.
 *
 * Why not just use the regular `ref(null)` and bind it to the element?
 *
 * 1. To avoid unwrapping template refs which maybe components. This hook will always
 *    give us the actual element being bound the the element, and not the component
 *    options.
 *
 * 2. In some cases where we need an up-to-date value of the ref node,
 *    from the consuming component, we can use this hook.
 *
 * @returns []
 */
function useRef() {
  var refEl = (0, _vue.ref)(null);
  (0, _vue.onBeforeUpdate)(function () {
    // clear refs before DOM updates
    refEl.value = null;
  });
  /**
   * Getter function to bind ref to value
   * @param el Template ref value provided by Vue
   */

  var _ref = function _ref(el) {
    var _$el;

    refEl.value = (_$el = el == null ? void 0 : el.$el) != null ? _$el : el;
  };

  return [_ref, refEl];
}
/** Vue Component HTML Element Instance */


/**
 * Unwraps element from ref
 * @param elementRef Ref of template node
 */
function unrefElement(elementRef) {
  var _$el2;

  var node = (0, _vue.unref)(elementRef);
  return (_$el2 = node == null ? void 0 : node.$el) != null ? _$el2 : node;
}
/**
 * Creates a ref whose value updates are debounced
 *
 * @example Simple example
 *
 * ```ts
 * const foo = useDebouncedRef('bar')
 * foo.value = 'baz'
 *
 * // foo.value to be updated to 'baz' after the delay of 300ms
 * ```
 *
 * @example Custom delay
 *
 * ```ts
 * const foo = useDebouncedRef('bar', 500)
 * foo.value = 'baz'
 *
 * // foo.value to be updated to 'baz' after the delay of 500ms
 * ```
 */


function useDebouncedRef(initialValue, delay, immediate) {
  if (delay === void 0) {
    delay = 300;
  }

  if (immediate === void 0) {
    immediate = false;
  }

  var state = (0, _vue.ref)(initialValue);
  var debouncedRef = (0, _vue.customRef)(function (track, trigger) {
    return {
      get: function get() {
        track();
        return state.value;
      },
      set: (0, _timers.debounce)(function (value) {
        state.value = value;
        trigger();
      }, delay, immediate)
    };
  });
  return debouncedRef;
}
//# sourceMappingURL=dom.js.map